/***********************************************************************/
/*                                                                     */
/*                      ADOBE CONFIDENTIAL                             */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                         */
/*                                                                     */
/*  Copyright 2016 Adobe Systems Incorporated                          */
/*  All Rights Reserved.                                               */
/*                                                                     */
/* NOTICE:  All information contained herein is, and remains           */
/* the property of Adobe Systems Incorporated and its suppliers,       */
/* if any.  The intellectual and technical concepts contained          */
/* herein are proprietary to Adobe Systems Incorporated and its        */
/* suppliers and are protected by all applicable intellectual property */
/* laws, including trade secret and copyright laws.                    */
/* Dissemination of this information or reproduction of this material  */
/* is strictly forbidden unless prior written permission is obtained   */
/* from Adobe Systems Incorporated.                                    */
/*                                                                     */
/***********************************************************************/

var gDbgLog = null;
var kLogFilename_Debug = 'onboarding_dbg.log';

function DebugLog(/*[String]*/ inPath, /*[Number]*/ inLogLevel, /*[Number]*/ inOutput)
{
	var path = inPath;
	var active = isValidProperty(inLogLevel) && inLogLevel != DebugLog.LOGLEVEL_NONE;
	var level = inLogLevel;
	var output = inOutput;
	var data = '';
	
	this.log = function(/*[String]*/ inWhat, /*[Number]*/ inLogType)
	{
		var type = isValidProperty(inLogType) ? inLogType : DebugLog.LOGLEVEL_NONE;

		if (active && type >= level)
		{
			var entry = '';
			entry += Date().toString();
			entry += '\t';

			switch(type)
			{
				case DebugLog.LOGLEVEL_INFO:	entry += '[INFO] ';		break;
				case DebugLog.LOGLEVEL_ERROR:	entry += '[ERROR] ';	break;
				case DebugLog.LOGLEVEL_FATAL:	entry += '[FATAL] ';	break;
			}

			entry += inWhat;
			entry += '\n';

			doLog(entry);
		}
	}

	function doLog(/*[String]*/ inWhat)
	{
		if (output & DebugLog.LOGOUTPUT_CONSOLE)
		{
			console.log(inWhat);
		}

		if (output & DebugLog.LOGOUTPUT_FILE)
		{
			store(inWhat);
		}

		if (output & DebugLog.LOGOUTPUT_ALERT)
		{
			window.alert(inWhat);
		}
	}
	
	function store(/*[String]*/ inWhat)
	{
		var ret = false;		
		var logFilePath = path + kLogFilename_Debug;
		
		createFolder(path);
		
		data += inWhat;
		result = cep.fs.writeFile(logFilePath, data);
		
		return (isValidProperty(result) && result.err == cep.fs.NO_ERROR);
	}
	
	function createFolder(/*[String]*/ inPath)
	{
		var result = cep.fs.stat(inPath);
		
		if (isValidProperty(result) && !result.data.isDirectory())
		{
			cep.fs.makedir(inPath);
		}
	}
	
	(function()
	{
		cep.fs.deleteFile(inPath + kLogFilename_Debug);
	})();
}

DebugLog.LOGLEVEL_NONE = 0;
DebugLog.LOGLEVEL_INFO = 2;
DebugLog.LOGLEVEL_ERROR = 4;
DebugLog.LOGLEVEL_FATAL = 8;

DebugLog.LOGOUTPUT_CONSOLE	= 0x01;
DebugLog.LOGOUTPUT_FILE		= 0x02;
DebugLog.LOGOUTPUT_ALERT	= 0x04;

DebugLog.get = function()
{
	if (!isValidProperty(gDbgLog))
	{
		var path = getPreferencesFolder() + 'logs/';
		gDbgLog = new DebugLog(path, __loglevel__, __logoutput__);
	}
	
	return gDbgLog;
}
