/***********************************************************************/
/*                                                                     */
/*                      ADOBE CONFIDENTIAL                             */
/*                   _ _ _ _ _ _ _ _ _ _ _ _ _                         */
/*                                                                     */
/*  Copyright 2016 Adobe Systems Incorporated                          */
/*  All Rights Reserved.                                               */
/*                                                                     */
/* NOTICE:  All information contained herein is, and remains           */
/* the property of Adobe Systems Incorporated and its suppliers,       */
/* if any.  The intellectual and technical concepts contained          */
/* herein are proprietary to Adobe Systems Incorporated and its        */
/* suppliers and are protected by all applicable intellectual property */
/* laws, including trade secret and copyright laws.                    */
/* Dissemination of this information or reproduction of this material  */
/* is strictly forbidden unless prior written permission is obtained   */
/* from Adobe Systems Incorporated.                                    */
/*                                                                     */
/***********************************************************************/

var externalObjectName = "PlugPlugExternalObject";
var lib = new ExternalObject( "lib:" + externalObjectName );

var eventObjects = {};
var eventCount = 0;

//////////////////////////////////////////////////////////////////////////////
//
// Dispatch ExtendScript events to CEP
// EventObjects ar copied (up to the first level of properties) for later access
//
function dispatchEvent(eventObj)
{
    eventObjects[eventCount] = copyEventObject(eventObj);

	//
	// send event to CEP
	//
    var event = new CSXSEvent();
    event.type = eventObj.type;
    event.data = eventCount;
    event.dispatch();

	// increase after dispatching event!
    eventCount++;
}

//////////////////////////////////////////////////////////////////////////////
//
// Clear stored copies of EventObjects
//
function clearEvents()
{
	eventObjects = {};
	eventCount = 0;

	return createResultJSON(false);
}

//////////////////////////////////////////////////////////////////////////////
//
// Return copy of EventObject with given index
//
function getEventObject(inIndex)
{
	var ret = null;
	
	if (inIndex < eventCount)
	{
		ret = eventObjects[inIndex];
	}
	
	return ret; 
}

//////////////////////////////////////////////////////////////////////////////
//
// Validate condition scriptlet
//
function validateCondition(inCondition, inEventObjIndex, inLog)
{
	var error = false;
	var data = [];
	var result = '';

    var triggercond = inCondition;
    var index = NaN;
    
    try
    {
    	index = parseInt(inEventObjIndex);
    }
    catch(e)
    {
    	index = NaN;
    }

    if (!isNaN(index))
	{
		var replacement = 'getEventObject(' + index + ')';

		while (triggercond.indexOf('${EVOBJ}$') >= 0)
		{
			triggercond = triggercond.replace('${EVOBJ}$', replacement);
		}
	}

    try
    {
        result = eval(triggercond);
    }
    catch(e)
    {
		error = true;
		data.push(e.toString());
    }

    if (error && isValidProperty(inLog) && inLog)
    {
		data.push(logCondition(index, inCondition, ret.result));
		data.push(logEventObj(index));
    }

    if (error)
	{
		if (data.length == 0)
		{
			data = '""';
		}
		else if (data.length == 1)
		{
			data = '"' + data[0] + '"';
		}
		else
		{
			var tmp = '[';

			for (var i=0; i<data.length; i++)
			{
				tmp += '"' + data[i] + '"';

				if (i+1 < data.length)
				{
					tmp += ",";
				}
			}

			tmp += ']';
			data = tmp;
		}
	}
	else
	{
		if (isValidProperty(result) && result.reflect.name == 'String')
		{
			data = '"' + result + '"';
		}
		else
		{
			data = result;
		}
	}

    return createResultJSON(error, data);
}
